package com.example.greenwatch;

import android.graphics.ImageFormat;
import android.media.Image;
import android.os.Bundle;
import android.os.Handler;
import android.util.Log;
import android.util.Size;

import android.widget.TextView;

import androidx.appcompat.app.AppCompatActivity;
import androidx.camera.core.CameraSelector;
import androidx.camera.core.ImageAnalysis;
import androidx.camera.lifecycle.ProcessCameraProvider;
// PREVIEW
//import androidx.camera.view.PreviewView;
import androidx.core.content.ContextCompat;

import com.google.common.util.concurrent.ListenableFuture;

import java.nio.ByteBuffer;
import java.util.concurrent.ExecutionException;

public class KameraAktivitaet extends AppCompatActivity {

    // PREVIEW
    // private PreviewView previewView;
    private boolean isMotionDetected;
    private boolean camera_alarm;
    private TextView alarm;
    private TextView test;


    // Bildverarbeitung Variablen
    private ByteBuffer previousBuffer;
    private int previousWidth;
    private int previousHeight;
    private int threshold = 50;
    private int startX;
    private int startY;
    private int endX;
    private int endY;

    private static final long ALARM_RESET_DELAY = 5000;
    private boolean isAlarmSet = false;
    private Handler alarmResetHandler = new Handler();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        // NUR FÜR TESTS
        alarm = findViewById(R.id.textView);
        test = findViewById(R.id.textView2);

        // PREVIEW //
        // previewView = findViewById(R.id.previewView);

        final ListenableFuture<ProcessCameraProvider> cameraProviderFuture = ProcessCameraProvider.getInstance(this);
        cameraProviderFuture.addListener(() -> {
            try {
                bindImageAnalysis(cameraProviderFuture.get());
            } catch (ExecutionException | InterruptedException e) {
                e.printStackTrace();
            }
        }, ContextCompat.getMainExecutor(this));
    }

    private void bindImageAnalysis(ProcessCameraProvider cameraProvider) {
        ImageAnalysis.Builder builder = new ImageAnalysis.Builder();
        builder.setTargetResolution(new Size(640, 480));
        builder.setBackpressureStrategy(ImageAnalysis.STRATEGY_KEEP_ONLY_LATEST);
        ImageAnalysis imageAnalysis = builder.build();
        imageAnalysis.setAnalyzer(
                ContextCompat.getMainExecutor(this),
                imageProxy -> {

                    int imageFormat = imageProxy.getFormat();

                    if (imageFormat == ImageFormat.YUV_420_888) {

                        Image currentImage = imageProxy.getImage();

                        if (previousHeight != 0) {
                            assert currentImage != null;
                            isMotionDetected = compareFrames(currentImage);
                        }

                        assert currentImage != null;
                        Image.Plane[] planes = currentImage.getPlanes();
                        ByteBuffer buffer = planes[0].getBuffer().duplicate();
                        previousBuffer = ByteBuffer.allocateDirect(buffer.remaining());
                        previousBuffer.put(buffer);
                        previousWidth = currentImage.getWidth();
                        previousHeight = currentImage.getHeight();

                        currentImage.close();

                        if (isMotionDetected) {
                            alarm.setText("ALARM");
                            camera_alarm = true;

                            alarmResetHandler.postDelayed(alarmResetRunnable, ALARM_RESET_DELAY);
                        }
                    }

                    imageProxy.close();
                });

        // Preview preview = new Preview.Builder().build();

        CameraSelector cameraSelector = new CameraSelector.Builder().requireLensFacing(CameraSelector.LENS_FACING_BACK).build();

        // PREVIEW //
        // preview.setSurfaceProvider(previewView.getSurfaceProvider());

        cameraProvider.bindToLifecycle(this, cameraSelector, imageAnalysis);

    }

    // Bildverarbeitung zur Bewegungserkennung
    private boolean compareFrames(Image currentImage) {
        Image.Plane[] planes = currentImage.getPlanes();
        ByteBuffer currentBuffer = planes[0].getBuffer();
        int currentWidth = currentImage.getWidth();
        int currentHeight = currentImage.getHeight();
        int yRowStride = planes[0].getRowStride();
        int yPixelStride = planes[0].getPixelStride();


        //System.out.println("Höhe: " + currentHeight + "   Breite: " + currentWidth);

        if (previousWidth != currentWidth || previousHeight != currentHeight) {
            return false;
        }

        // Blöcke weiter verkleinern
        int blockSize = kleinstesQuadrat(previousHeight, previousWidth) / 8;

        //System.out.println(blockSize);

        int numBlocksX = currentWidth / blockSize;
        int numBlocksY = currentHeight / blockSize;

        for (int blockY = 0; blockY < numBlocksY; blockY++) {
            for (int blockX = 0; blockX < numBlocksX; blockX++) {
                startX = blockX * blockSize;
                startY = blockY * blockSize;
                endX = startX + blockSize;
                endY = startY + blockSize;

                float currentLuminance = berechneMittlereLuminanz(currentBuffer, yRowStride, yPixelStride);
                float previousLuminance = berechneMittlereLuminanz(previousBuffer, yRowStride, yPixelStride);

                int pixelDifference = Math.abs((int) previousLuminance - (int) currentLuminance);

                //System.out.println(pixelDifference);

                if (pixelDifference > threshold) {
                    System.out.println(pixelDifference);
                    return true;
                }
            }
        }

        return false;
    }

    private float berechneMittlereLuminanz(ByteBuffer buffer, int rowStride, int pixelStride) {
        int sumLuminance = 0;
        int countPixels = 0;

        for (int row = startY; row < endY; row++) {
            for (int col = startX; col < endX; col++) {
                int bufferIndex = row * rowStride + col * pixelStride;
                int currentPixel = buffer.get(bufferIndex) & 0xFF;

                sumLuminance += currentPixel;
                countPixels++;
            }
        }

        float averageLuminance = (float) sumLuminance / countPixels;

        return averageLuminance;
    }

    private int kleinstesQuadrat(int height, int width) {
        if (height == width) {
            return height;
        } else if (height > width) {
            return kleinstesQuadrat(height - width, width);
        } else {
            return kleinstesQuadrat(height, width - height);
        }
    }

    private Runnable alarmResetRunnable = new Runnable() {
        @Override
        public void run() {
            camera_alarm = false;
            alarm.setText("OK");
        }
    };
}

